@@ -24,23 +24,32 @@ void about_destroy_dialog(GtkWidget *okno, gpointer dane) ;
 void about_close_dialog(GtkWidget *kontrolka, gpointer dane) ;
 
 
-/* we fork in help_display man, so we have to set the SICHLD signal */
-void help_display_man_catch_child(int signal) {
-	int child_status ;
-	
-	wait(&child_status) ;
+void help_short_info(char *progname) {
+
+	fprintf(stderr, 
+		"%s - graphical user interface for command line programs\n"
+		"  Usage:\n"
+		"     %s [options] [ input file ] [ output file ]\n"
+		"\n"
+		"  Options:\n"
+		"     -d       : turn debugging on\n"
+		"     -v       : print version information & exit\n"
+		"     -h       : display this short help message\n\n"
+		" Note that %s needs a graphical display\n\n", 
+		PROGRAM, progname, PROGRAM);
+	return ;
 }
 
 
 /* displays the manual page for a specific page */
 int help_display_man_lowlevel(char *name) {
 	char *command ;
-	int res, pid ;
 
-	Warn("displaying man page for %s", name) ;
+	say(0, "displaying man page for %s", name) ;
 	command = malloc(strlen(name) + 50) ;
 	sprintf(command, "xterm -e man %s &", name) ;
 	system(command) ;
+	free(command) ;
 	return EXIT_SUCCESS ;
 }
 
@@ -56,7 +65,7 @@ void help_display_man(GtkWidget *kontr, gpointer dane) {
 
 /* display a help text */
 void help_text_lowlevel(char *textstring) {
-	GtkWidget *okno, *tekst, *twin, *hpole, *przycisk ;
+	GtkWidget *okno, *tekst, *twin, *przycisk ;
 	int i = 0 ;
 
 	okno = gtk_dialog_new() ;
@@ -96,7 +105,6 @@ void help_text_lowlevel(char *textstring) {
 /* display a help text */
 void help_text(gpointer dane, guint signal, GtkWidget *kontrolka) {
 	char *textstring ;
-	int i = 0 ;
 
 	switch(signal){
 		case 1: textstring = license ;
