@@ -11,26 +11,26 @@
 #include <string.h>
 #include <time.h>
 #include <gtk/gtk.h>
+#include "gp_getopt.h"
 #include "main.h"
-#include "genpak.h"
+//#include "genpak.h"
 
 
 opt_s opt ;
 
 /* writes a text into a specified text window or, if window is not currently
  * displayed, into the apripriate buffer */
-int write_text(char *text, okno_s *okno, opt_s *o) {
+int write_text(char *text, okno_s *okno, opt_s *o, int color) {
 
 	if(!text) return EXIT_FAILURE;
 
-	gtk_text_insert(GTK_TEXT(okno->tekst), o->czcionka, NULL, NULL, text, -1) ;
+	gtk_text_insert(GTK_TEXT(okno->tekst), o->czcionka, &o->colors[color], NULL, text, -1) ;
 	gtk_text_insert(GTK_TEXT(okno->tekst), o->czcionka, NULL, NULL, "\n", -1) ;
 			
 	return EXIT_SUCCESS ;
 }
 
 
-
 /* kills a process. Called from a button widget */
 void killer(GtkWidget *kontrolka, gpointer dane) {
 	GtkWidget * window ;
@@ -41,26 +41,24 @@ void killer(GtkWidget *kontrolka, gpointer dane) {
 	pid = (int *) d->data1 ;
 	window = d->window1 ;
 
-	Warn("killing PID %i", *pid) ;
+	say(0, "killing PID %i", *pid) ;
 
 	if(kill(*pid, SIGKILL)) 
-		Warn("Could not kill the process") ;
+		say(0, "Could not kill the process") ;
 	else
-		Warn("Process %i KIA", *pid) ;
+		say(0, "Process %i KIA", *pid) ;
 	
 	free(d) ;
 		
 }
 
 /* creates a window with a big red button on it :-) */
-GtkWidget* executor_abort(char *comline, int *pid) {
+GtkWidget* executor_abort(char *comline, int *pid, opt_s *o) {
 	GtkWidget * res, *label, *abort ; 
 	char *labeltext ;
 	data_s *d ;
 
-
-	Warn("creating `abortion box'") ;
-
+	say(0, "creating `abortion box'") ;
 	res = gtk_dialog_new() ;
 
 	labeltext = malloc(strlen(comline) + 200) ;
@@ -82,10 +80,10 @@ GtkWidget* executor_abort(char *comline, int *pid) {
 	gtk_widget_grab_default(abort) ;
 
 	gtk_widget_show_all(res) ;
+	gdk_window_set_cursor(res->window, o->cursor_wait) ;
 	gtk_grab_add(res) ;
 	free(labeltext) ;
 	return res ;
-
 }
 
 
@@ -107,16 +105,24 @@ int executor(char *comline, opt_s *o) {
 			return EXIT_FAILURE ;
 		case 0: /* child process */
 			exit = system(comline) ;
-			Warn("OK, system() terminated, exit status %i", exit) ;
+
+			if(WIFEXITED(exit)) {
+				exit = WEXITSTATUS(exit) ;
+				say(0, "executor: child: exited normally, %i", exit) ;
+			} else {
+				say(0, "executor: child: exited abnormally, code %i", exit) ;
+			}
+				
 			_exit(exit) ;
 			break;
 		default: /* parent */
-			terminator = executor_abort(comline, &pid) ;
+			terminator = executor_abort(comline, &pid, o) ;
 
 			/* refreshing dialog while waiting for the child to exit */
 			while( (exitpid = waitpid(pid, &status, WNOHANG)) <= 0) {
 				g_main_iteration(FALSE) ;
 			}
+
 			gtk_widget_destroy(terminator) ;
 			return status ;
 			break ;
@@ -131,13 +137,17 @@ int executor(char *comline, opt_s *o) {
 int execute(char *comline, opt_s *o) {
 
 	char *tmp, *real;
-	int status, exit_value ;
+	int color = 0, status, exit_value ;
 	FILE *fp ;
 
 	if(!comline) return EXIT_FAILURE ;
 	gdk_window_set_cursor(o->okno->window, o->cursor_wait) ;
 
-	if(window_tofile(&o->panels[0], o) || write_text(comline, &o->panels[2], o)) {
+	/* filling in file parameters and executing the command */
+	o->command_history = g_list_prepend(o->command_history, comline) ;
+
+	if(window_tofile(&o->panels[0], o) || 
+		write_text(comline, &o->panels[2], o, 3)) {
 		gdk_window_set_cursor(o->okno->window, o->cursor_normal) ;
 		return EXIT_FAILURE ;
 	}
@@ -146,7 +156,7 @@ int execute(char *comline, opt_s *o) {
 	tmp = my_strsubst(comline, "%input", o->panels[0].tmpfil) ;
 	real = malloc(strlen(tmp) + 2 * strlen(o->panels[1].tmpfil) + 10) ;
 	sprintf(real, "%s > %s 2> %s", tmp, o->panels[1].tmpfil, o->panels[2].tmpfil) ;
-	Warn("executing %s", real) ;
+	say(0, "executing %s", real) ;
 	
 	/* OK, let's go */
 	status = executor(real, o) ;
@@ -158,21 +168,23 @@ int execute(char *comline, opt_s *o) {
 
 	/* checking whether child exited normally */
 	if(!WIFEXITED(status)) {
-		Warn("Child process exited abnormally.") ;
+		say(0, "Child process exited abnormally.") ;
 		return EXIT_FAILURE ;
 	}
 
 	switch( (exit_value = WEXITSTATUS(status)) ) { 
 		case 127:
-			komunikat("/bin/sh is missing (how can it be?)") ;
+			komunikat("/bin/sh is missing or program not found") ;
 			return EXIT_FAILURE ;
 			break ;
 		case 0:
+			color = 2 ; /* green color */
 			break ;
 		default:
 			komunikat("There was an error executing your command.\n"
 				"system() returned %i", exit_value) ;
-			write_text("Error during execution\n", &o->panels[2], o) ;
+			write_text("Error during execution\n", &o->panels[2], o, 1) ;
+			color = 1 ; /* red */
 			break ;
 	}
 
@@ -182,7 +194,7 @@ int execute(char *comline, opt_s *o) {
 		return EXIT_FAILURE;
 	}
 
-	read_file(fp, o, &o->panels[2]) ;
+	read_file(fp, o, &o->panels[2], color) ;
 	fclose(fp) ;
 
 	if( (fp = fopen(o->panels[1].tmpfil, "r")) == NULL) {
@@ -191,7 +203,7 @@ int execute(char *comline, opt_s *o) {
 		return EXIT_FAILURE;
 	}
 
-	read_file(fp, o, &o->panels[1]) ;
+	read_file(fp, o, &o->panels[1], -1) ;
 	fclose(fp) ;
 	return EXIT_SUCCESS ;
 }
@@ -274,11 +286,8 @@ void command(gpointer dane, guint signal, GtkWidget *kontrolka) {
 		return ;
 	}
 
-	/* filling in file parameters and executing the command */
-	o->command_history = g_list_prepend(o->command_history, polec) ;
-
 	execute(polec, o) ;
-	Warn("command dialog finished") ;
+	say(0, "command dialog finished") ;
 }
 
 
@@ -287,12 +296,12 @@ void signal_handler(int signal) {
 
 	switch(signal){
 		case SIGINT:
-			Warn("Keyboard interrupt caught, exiting") ;
+			say(0, "Keyboard interrupt caught, exiting") ;
 			break ;
 		case SIGSEGV:
 			fprintf(stderr, "\nMemory protection fault (SIGSEGV caught).\n"
-			"Trying to exit gracefully: saving configuration.\n"
 			"Please record Arka messages and mail them to january@bioinformatics.org\n\n") ;
+			exit(EXIT_FAILURE) ;
 			break ;
 		default:
 			break ;
@@ -304,11 +313,43 @@ void signal_handler(int signal) {
 }
 
 
+/* parses command line options */
+int parse_command_line(int argc, char *argv[]) {
+	char c ;
+	extern int optind ;
+
+
+	say(0, "Parsing command line options") ;
+
+	while ((c = gp_getopt(argc, argv, "dvh")) != EOF)
+		switch(c) {
+		case 'd':
+			debug = TRUE ;
+			say(0, "Running in debug mode") ;
+			break ;
+		case 'v':
+			fprintf(stderr, "%s, version %s\n", PROGRAM, VERSION) ;
+			exit(EXIT_SUCCESS) ;
+		case 'h':
+			help_short_info(PROGRAM) ;
+			exit(EXIT_SUCCESS) ;
+		default:
+			fprintf(stderr, "%s: wrong option '%c'\n", argv[0], c) ;
+			fprintf(stderr, "Type `%s -h' or choose Help/Readme to get help\n", argv[0]) ;
+			komunikat("%s: wrong option '%c'\n"
+				"Choose Help/Readme to get help", argv[0], c) ;
+			break;
+		}
+
+	return optind ;
+}
+
+
 /*
  * main function -- short, what?
  */
 int main(int argc, char *argv[]) {
-	int i ;
+	int optind, i ;
 	GtkWidget *popup ;
 
 	if(!gtk_init_check(&argc, &argv)) {
@@ -320,13 +361,18 @@ int main(int argc, char *argv[]) {
 		exit(EXIT_FAILURE);
 	}
 
+	memory_start() ;
 
 	/* the wee little window shown at the start of the program */
 	popup = popup_at_start(&opt) ;
+	while(gtk_events_pending()) gtk_main_iteration() ;
 
 	/* initialize the options structure */
 	options_initialize(&opt) ;
 
+	/* reading command line options */
+	optind = parse_command_line(argc, argv) ;
+
 	/* reading the configuration file to update more options */
 	read_rcfile(&opt) ;
 
@@ -342,6 +388,22 @@ int main(int argc, char *argv[]) {
 	signal(SIGINT, signal_handler) ;
 	signal(SIGSEGV, signal_handler) ;
 
+	while(gtk_events_pending()) gtk_main_iteration() ;
+	/* read file from command line input */
+	if(optind < argc) {
+		printf("One left\n") ;
+		printf("%s\n", argv[optind] ) ;
+		file_open_from_name(&opt, 0, argv[optind]) ;
+	}
+
+	say(0, "looking for the next one") ;
+
+	if( (optind + 1) < argc) {
+		printf("Still one left\n") ;
+		printf("%s\n", argv[optind + 1] ) ;
+		//opt.panels[1].filenam = my_strdup(argv[optind + 1]) ;
+	}
+
 	/* starting the main gtk loop */
 	gtk_main() ;
 
