@@ -1,6 +1,6 @@
 #define PROGRAM "arka"
-#define VERSION "0.06"
-#define ARKA_PROG_NUM_MAX 200
+#define VERSION "0.10"
+#include "genpak.h"
 
 #ifndef ARKA_PROGRAMS
 	#define ARKA_PROGRAMS "/etc/arkarc"
@@ -15,22 +15,35 @@
 	#define ARKA_GLOBAL "/etc/arkarc"
 #endif
 
+#define g_free(smth) arka_free(__FILE__, __LINE__, smth)
+#define g_malloc(smth) arka_malloc(__FILE__, __LINE__, smth)
+#define g_realloc(arg1,arg2) arka_realloc(__FILE__, __LINE__, arg1,arg2)
+#define free(smth) arka_free(__FILE__, __LINE__, smth)
+#define malloc(smth) arka_malloc(__FILE__, __LINE__, smth)
+#define realloc(arg1,arg2) arka_realloc(__FILE__, __LINE__, arg1,arg2)
+
+/* typedef enum {UNKNOWN=0, DNA, RNA, PROTEIN, WRONG } sequence_type ; */
+typedef enum {INPUT=0, OUTPUT, ERROR } panels_type ;
 
-typedef enum {UNKNOWN=0, DNA, RNA, PROTEIN, WRONG } sequence_type ;
 #ifdef MAIN_C
 	int debug = 0 ;
+	int quiet = 0 ;
+	int html = 0 ;
 #else
 	extern int debug ;
+	extern int quiet ;
+	extern int html ;
 #endif
 
+/*
 struct sekw_ss {
 	char name[200] ;
 	char *sequ ;
 	long leng ;
 	struct sekw_ss *next ;
-	sequence_type type ; } ;
+	sequence_type type ; } ; */
 
-typedef struct sekw_ss sekw ;
+/* typedef struct sekw_ss sekw ; */
 
 typedef struct {
 	GtkWidget * window ;
@@ -49,18 +62,22 @@ typedef struct {
 /* structure containing a frame with a text widget and two scrollbars */
 typedef struct {
 	int on ;
+	int editable ;
 	int changed ;
 	int overwrite ;
+
 	char file ;
 	char *filenam ;
 	char tmpfil[FILENAME_MAX] ;
 	char title[100] ;
+
 	gchar *contents ;
 	GtkWidget *parent ;
 	GtkWidget *ramka ;
 	GtkWidget *tabela ;
 	GtkWidget *tekst ;
 	GtkWidget *tmp ; } okno_s ;
+
 	
 /* structure holding all data about the current state of things */
 typedef struct {
@@ -92,8 +109,13 @@ typedef struct {
 	int file_history_size ; /* how many file history items to save */
 
 	int editable ; /* is the input editable? */
+	char *external_editor ; /* command for external editor */
+
 	int show_tooltips ; /* show tooltips? */
 
+	int sequence_editor_window_opened ;
+	int search_and_replace_dialog_opened ;
+
 	GtkWidget *tmp ; /* temporary data */
 } opt_s ;
 
@@ -101,6 +123,12 @@ typedef struct {
 /* types of options */
 typedef enum { BOOLEAN, INT, DOUBLE, STRING, FILENAME, HIDDEN } opttypes_t ;
 
+
+/* shortcuts for colors */
+typedef enum { 
+	BLACK, RED, GREEN, BLUE, VIOLET, YELLOW, MAGENTA, CYAN, WHITE 
+} color_e ;
+
 /* structure holding options for one of the program entries */
 typedef struct {
 	GtkWidget *widget ;
@@ -163,27 +191,37 @@ void inicjuj_main_window(opt_s *o) ;
 void toggle_windows(gpointer dane, guint signal, GtkWidget *kontrolka) ;
 int append_single_entry(opt_s *o, char *filen, int i) ;
 char *get_font_name(opt_s *o, char *name) ;
+int allocate_colors(opt_s *o) ;
+
 
   
 /* graph.c */
 void graph_gc(gpointer dane, guint signal, GtkWidget *kontrolka) ;
 
 /* main.c */
-int write_text(char *text, okno_s *okno, opt_s *o) ;
+int write_text(char *text, okno_s *okno, opt_s *o, int color) ;
 int execute(char *comline, opt_s *o) ;
 void command(gpointer dane, guint signal, GtkWidget *kontrolka) ;
 
+/* misc.c */
+char *my_strdup(char *string) ;
+char *my_strsubst(char *string, char *from, char *to) ;
+void say(int level, char *msg, ...) ;
+void die(char *msg, ...) ;
+
 /* graph3d.c */
 void graph3d_create_from_file(gpointer dane, guint signal, GtkWidget *kontrolka) ;
 void graph3d_create_from_input(gpointer dane, guint signal, GtkWidget *kontrolka) ;
 
 /* file.c */
-int read_file(FILE *fp, opt_s *o, okno_s *okno) ;
+int print_file(FILE *fp, opt_s *o, okno_s *okno) ;
+int read_file(FILE *fp, opt_s *o, okno_s *okno, int color) ;
+int file_to_window(opt_s *o, okno_s *win, int color) ;
 char *file_get_name(opt_s *o, char *default_name) ;
 void file_select(gpointer dane, guint signal, GtkWidget *kontrolka) ;
-void save_file(GtkWidget *k, gpointer dane) ;
-void open_file(GtkWidget *k, gpointer dane) ;
+int file_open_from_name(opt_s *o, int signal, char *filenam) ;
 int window_tofile(okno_s *okno, opt_s *o) ;
+char *file_save_lowlevel(opt_s *o, int signal, char *filenam) ;
 void file_save_as(gpointer dane, guint signal, GtkWidget *kontrolka) ;
 void file_save(gpointer dane, guint signal, GtkWidget *kontrolka) ;
 void file_open(gpointer dane, guint signal, GtkWidget *kontrolka) ;
@@ -199,15 +237,15 @@ int read_rcfile(opt_s *o) ;
 int write_rcfile(opt_s *o) ;
 
 /* programs.c */
-char *my_strdup(char *string) ;
-char *my_strsubst(char *string, char *from, char *to) ;
 void program_create(gpointer dane, guint signal, GtkWidget *kontrolka) ;
 void program_select(gpointer dane, guint signal, GtkWidget *kontrolka) ;
 void programs_append(opt_s *o) ;
 char* set_options(int wh, opt_s *o) ;
 GtkWidget* set_widget_for_option(entry_option_s *o, GtkWidget *parentbox) ;
+int free_entries_all(opt_s *o) ;
 
 /* help.c */
+void help_short_info(char *filename) ;
 void help_display_man(GtkWidget *kontr, gpointer dane) ;
 void help_text(gpointer dane, guint signal, GtkWidget *kontrolka) ;
 void help_about(gpointer dane, guint signal, GtkWidget *kontrolka) ;
@@ -216,3 +254,16 @@ GtkWidget * popup_at_start(opt_s *o) ;
 /* options.c */
 int options_initialize(opt_s *opt) ;
 void options_preferences_set(gpointer dane, guint signal, GtkWidget *kontrolka) ;
+
+/* sequed.c */
+void seqed_start(gpointer dane, guint signal, GtkWidget *kontrolka) ;
+
+/* search.c */
+void search(gpointer dane, guint signal, GtkWidget *kontrolka) ;
+
+/* memory.c */
+void *arka_malloc(char *str, int line, size_t size) ;
+void *arka_realloc(char *str, int line, void *ptr, size_t size) ;
+void arka_free(char *str, int line, void *ptr) ;
+void memory_start() ;
+void memory_summary() ;
