@@ -21,7 +21,8 @@ typedef struct {
 entry_option_s options[] = {
 	{ NULL, NULL, "command history items to store", "20", 1, NULL, 0.0, 20, 1, INT, NULL } ,
 	{ NULL, NULL, "file history items to store", "5", 1, NULL, 0.0, 5, 1, INT, NULL } ,
-	{ NULL, NULL, "hide tooltips", "0", 0, NULL, 0.0, 0, 0, BOOLEAN, NULL } 
+	{ NULL, NULL, "hide tooltips", "0", 0, NULL, 0.0, 0, 0, BOOLEAN, NULL },
+	{ NULL, NULL, "external editor command", NULL, 0, NULL, 0.0, 0, 0, STRING, NULL } 
 } ;
 
 
@@ -81,6 +82,39 @@ int options_initialize(opt_s *opt) {
 	opt->cursor_normal = gdk_cursor_new(GDK_ARROW) ;
 	opt->cursor_wait = gdk_cursor_new(GDK_WATCH) ;
 
+	opt->external_editor = my_strdup("xterm -e vi %input") ;
+
+	opt->sequence_editor_window_opened = FALSE ;
+	opt->search_and_replace_dialog_opened = FALSE ;
+
+	return EXIT_SUCCESS ;
+}
+
+
+/* adjustst the current settings to the options */
+int options_reload(opt_s *o) {
+	GtkWidget *w ;
+	int i ;
+
+	/* setting hide / display tooltips */
+  w = gtk_item_factory_get_widget(o->fabryka, "/Options/Hide tooltips") ;
+
+	i = o->show_tooltips ;
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), !o->show_tooltips) ;
+	o->show_tooltips = i ;
+
+  if(o->show_tooltips) gtk_tooltips_enable(o->tips) ;
+  else gtk_tooltips_disable(o->tips) ;
+
+	/* setting fontname */
+	if(o->font_name) {
+		o->czcionka = gdk_font_load(o->font_name) ;
+	} else {
+		if(o->font_name_default) {
+			o->czcionka = gdk_font_load(o->font_name_default) ;
+		}
+	}
+
 	return EXIT_SUCCESS ;
 }
 
@@ -97,7 +131,7 @@ void options_get_font(GtkWidget *kontr, gpointer dane) {
 
 	fontname = get_font_name(o, o->font_name) ;
 
-	Warn("setting default font name to %s", fontname) ;
+	say(0, "setting default font name to %s", fontname) ;
 
 	if(fontname) {
 		gtk_label_set_text(GTK_LABEL(pref->font_label), fontname) ;
@@ -105,7 +139,6 @@ void options_get_font(GtkWidget *kontr, gpointer dane) {
 		o->font_name_default = fontname ;
 		if(o->font_name) free(o->font_name) ;
 		o->font_name = my_strdup(fontname) ;
-
 		o->czcionka = gdk_font_load(fontname) ;
 	}
 
@@ -116,27 +149,41 @@ void options_get_font(GtkWidget *kontr, gpointer dane) {
 void options_preferences_close(GtkWidget *kontr, gpointer dane) {
 	preferences_s *pref ;
 	opt_s * o ;
+	char *tmp ;
 
 	pref = (preferences_s *) dane ;
 
 	o = pref->o ;
 
+	/* read history sizes */
 	o->command_history_size =
 		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(options[0].widget)) ;
 	o->file_history_size =
 		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(options[1].widget)) ;
 
-	if(options[1].widget) {
+	/* external editor */
+	say(0, "getting external editor") ;
+	tmp = gtk_editable_get_chars(GTK_EDITABLE(options[3].widget), 0, -1) ;
+	say(0, "got: %s", tmp) ;
+
+	if(tmp) {
+		if(o->external_editor) g_free(o->external_editor) ;
+		o->external_editor = tmp ;
+		say(0, "external editor command: %s") ;
+	}
+
+	if(options[2].set) {
+		say(0, "hiding tooltips") ;
 		o->show_tooltips = FALSE ;
 		gtk_tooltips_disable(o->tips) ;
 	} else {
+		say(0, "showing tooltips") ;
 		o->show_tooltips = TRUE ;
 		gtk_tooltips_enable(o->tips) ;
 	}
 
 	gtk_widget_destroy(pref->okno) ;
 	
-	gtk_main_quit() ;
 	write_rcfile(o) ;
 	redraw_text(o) ;
 }
@@ -144,17 +191,19 @@ void options_preferences_close(GtkWidget *kontr, gpointer dane) {
 
 /* setting up the preferences dialog */
 void options_preferences_set(gpointer dane, guint signal, GtkWidget *kontrolka) {
-	GtkWidget *sep, *hpole, *ypole, *label, *notebook, *ok, *cancel ;
+	GtkWidget *hpole, *ypole, *label, *notebook, *ok ;
 	preferences_s pref ;
 	int nopts, i ;
 
 	nopts = sizeof(options) / sizeof(entry_option_s) ;
   pref.o = (opt_s *) dane ;
 
-	Warn("main preferences dialog") ;
+	say(0, "main preferences dialog") ;
 	
 	pref.okno = gtk_window_new(GTK_WINDOW_DIALOG) ;
 	gtk_window_set_title(GTK_WINDOW(pref.okno), "Preferences") ;
+	gtk_signal_connect(GTK_OBJECT(pref.okno), "destroy",
+		GTK_SIGNAL_FUNC(zakoncz_dialog), pref.okno) ;
 	ypole = gtk_vbox_new(FALSE, 5) ;
 
 	label = gtk_label_new("Main") ;
@@ -167,6 +216,7 @@ void options_preferences_set(gpointer dane, guint signal, GtkWidget *kontrolka)
 	options[0].intvalue = pref.o->command_history_size ;
 	options[1].intvalue = pref.o->file_history_size ;
 	options[2].set = !(pref.o->show_tooltips) ;
+	options[3].strvalue = pref.o->external_editor ;
 
 	/* same mechanism as in the programs menu -- to recycle the code */
 	for(i = 0 ; i < nopts ; i++) {
