@@ -36,58 +36,6 @@ int rebuild_menu(opt_s *o) ;
 
 /* print an entry on stdout */
 int entry_print(entry_s *e) ;
-char *my_strdup(char *string) ;
-
-/* -------------- some small, useful string funtions ------------- */
-
-/* ANSI C doesn't have a strdup */
-char *my_strdup(char *string) {
-	int i ;
-	char *res = NULL ;
-
-	if(!string) return NULL ;
-
-	i = strlen(string) ;
-	res = malloc(i + 1) ;
-	if(!res) return NULL ;
-
-	strcpy(res, string) ;
-	return res ;
-}
-
-
-/* exchange all occurencies of 'from' in 'string' by 'to' */
-char *my_strsubst(char *string, char *from, char *to) {
-	char *tmp, *t, *p, *res ;
-
-	res = NULL ;
-	tmp = my_strdup(string) ;
-	t = tmp ;
-
-	if(!string || !from || !to) return NULL ;
-
-	while( (p = strstr(t, from)) ) {
-		*p = '\0' ;
-		if(!res) {
-			res = malloc(strlen(t) + strlen(to) + 1) ;
-			*res = '\0' ;
-		} else res = realloc(res, strlen(res) + strlen(t) + strlen(to) + 1) ;
-		strcat(res, t) ;
-		strcat(res, to) ;
-		t = p + strlen(from) ;
-	}
-
-	if(!res) {
-		res = malloc(strlen(t) + 1) ;
-		strcpy(res, t) ;
-	} else {
-		res = realloc(res, strlen(res) + strlen(t) + 1) ;
-		strcat(res, t) ;
-	}
-
-	free(tmp) ;
-	return res ;
-}
 
 /* ----------------------- freeing memory ------------------------------------- */
 
@@ -131,6 +79,21 @@ int free_entry_s(entry_s *e, int signal) {
 }
 
 
+int free_entries_all(opt_s *o) {
+	int i ;
+	entry_s *p ;
+
+	for(i = 0 ; i < g_list_length(o->program_list) ; i++) {
+		p = g_list_nth_data(o->program_list, i) ;
+		say(0, "free_entries_all: freeing %s", p->menupath) ;
+		free_entry_s(p, 0) ;
+	}
+
+	g_list_free(o->program_list) ;
+
+	return EXIT_SUCCESS ;
+}
+
 /* creates either a new, empty entry_option_s structure or 
  * a clone of an existing struct */
 entry_option_s *new_entry_option_s(entry_option_s *o) {
@@ -139,7 +102,7 @@ entry_option_s *new_entry_option_s(entry_option_s *o) {
 	res = malloc(sizeof(*res)) ;
 
 	if(!res) {
-		Warn("new_entry_option_s: cannot allocate memory") ;
+		say(0, "new_entry_option_s: cannot allocate memory") ;
 		return NULL ;
 	}
 
@@ -172,6 +135,7 @@ entry_option_s *new_entry_option_s(entry_option_s *o) {
 	return res ;
 }
 
+
 /* creates either a new, empty entry_s structure or a clone of an existing struct */
 entry_s *new_entry_s(entry_s* e) {
 	entry_option_s *o1, *o2 ;
@@ -181,7 +145,7 @@ entry_s *new_entry_s(entry_s* e) {
 	res = malloc(sizeof(*res)) ;
 
 	if(!res) {
-		Warn("new_entry_s: cannot allocate memory") ;
+		say(0, "new_entry_s: cannot allocate memory") ;
 		return NULL ;
 	}
 
@@ -194,7 +158,7 @@ entry_s *new_entry_s(entry_s* e) {
 		res->branch = NULL ;
 		res->menupath = NULL ;
 	} else { /* copying provided element */
-		Warn("new_entry_s: copying %s", e->name) ;
+		say(0, "new_entry_s: copying %s", e->name) ;
 		res->name = my_strdup(e->name) ;
 		res->entry = my_strdup(e->entry) ;
 		res->options = NULL ;
@@ -373,14 +337,14 @@ int print_program_tofile(FILE *fp, entry_s *p) {
 int extract_entries_from_program(entry_s *p, GtkWidget** e) {
 	int i ;
 
-	Warn("extract_entries_from_program: deleting widget contents") ;
+	say(0, "extract_entries_from_program: deleting widget contents") ;
 	gtk_editable_delete_text(GTK_EDITABLE(e[0]), 0, -1) ;
 	gtk_editable_delete_text(GTK_EDITABLE(e[1]), 0, -1) ;
 	gtk_editable_delete_text(GTK_EDITABLE(e[2]), 0, -1) ;
 	gtk_editable_delete_text(GTK_EDITABLE(e[3]), 0, -1) ;
 	gtk_editable_delete_text(GTK_EDITABLE(e[4]), 0, -1) ;
 
-	Warn("extract_entries_from_program: extracting widgets from program data") ;
+	say(0, "extract_entries_from_program: extracting widgets from program data") ;
 
 	
 	i = 0 ;
@@ -400,7 +364,7 @@ int extract_entries_from_program(entry_s *p, GtkWidget** e) {
 	if(p->long_description) 
 		gtk_editable_insert_text(GTK_EDITABLE(e[4]), p->long_description,
 			strlen(p->long_description), &i) ;
-	Warn("done") ;
+	say(0, "done") ;
 
 	return EXIT_SUCCESS ;
 }
@@ -409,7 +373,7 @@ int extract_entries_from_program(entry_s *p, GtkWidget** e) {
 /* used in the dialog to get the program description from widgets */
 int extract_program_from_entries(entry_s *p, GtkWidget** e) {
 
-	Warn("extracting program data from widgets") ;
+	say(0, "extracting program data from widgets") ;
 	p->name = gtk_editable_get_chars(GTK_EDITABLE(e[0]), 0, -1) ;
 
 	/* only allow to proceede if program name defined */
@@ -417,7 +381,7 @@ int extract_program_from_entries(entry_s *p, GtkWidget** e) {
 		komunikat("You have to enter at least the p name") ;
 		g_free(p->name) ; 
 		return EXIT_FAILURE ; 
-	} else Warn("name is %s", p->name) ;
+	} else say(0, "name is %s", p->name) ;
 
 	p->entry = gtk_editable_get_chars(GTK_EDITABLE(e[1]), 0, -1) ;
 
@@ -468,7 +432,7 @@ void programs_save(GtkWidget *kontr, gpointer dane) {
 		filename = realloc(filename, strlen(filename) + strlen(ARKA_DIR) + 20) ;
 		strcat(filename, ARKA_DIR) ;
 		strcat(filename, "/programs") ;
-		Warn("Saving spec for program %s to file %s", p->name, filename) ;
+		say(0, "Saving spec for program %s to file %s", p->name, filename) ;
 		fp = fopen(filename, "a") ;
 		free(filename) ;
 	}
@@ -534,7 +498,7 @@ GtkWidget* option_create_combo(GtkWidget *parent, entry_s *p) {
 	}
 
 	/* the combo box */
-	Warn("option_create_combo: creating combo box") ;
+	say(0, "option_create_combo: creating combo box") ;
   res = gtk_combo_new() ;
   gtk_box_pack_start(GTK_BOX(parent), res, FALSE, FALSE, 5) ;
 	gtk_box_reorder_child(GTK_BOX(parent), res, 1) ;
@@ -549,7 +513,7 @@ GtkWidget* option_create_combo(GtkWidget *parent, entry_s *p) {
 	}
 
 	g_list_free(l) ;
-	Warn("option_create_combo: done") ;
+	say(0, "option_create_combo: done") ;
 	return res ;
 }
 
@@ -582,7 +546,7 @@ int update_option_dialog(gpointer dane) {
 	gtk_label_set_text(GTK_LABEL(dialog[0].widget), tmp) ;
 	free(tmp) ;
 	
-	Warn("update_option_dialog: done") ;
+	say(0, "update_option_dialog: done") ;
 	return EXIT_SUCCESS ;
 }
 
@@ -596,7 +560,7 @@ void delete_option(GtkWidget *kontr, gpointer dane) {
 	int number ;
 	char *tmp ;
 
-	Warn("deleting option...") ;
+	say(0, "deleting option...") ;
 	data = (data_s *) dane ;
 	program = (entry_s*) data->data1 ;
 	dialog = (entry_option_s*) data->data2 ;
@@ -605,7 +569,7 @@ void delete_option(GtkWidget *kontr, gpointer dane) {
 	tmp = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(data->window2)->entry), 0, -1) ;
 
 	if(sscanf(tmp, "%i -", &number) != 1) {
-		Warn("reading option combo failed. Sorry.") ;
+		say(0, "reading option combo failed. Sorry.") ;
 	} else {
 		o = g_list_nth_data(program->options, (number - 1)) ;
 		program->options = g_list_remove(program->options, o) ;
@@ -634,10 +598,10 @@ void edit_option(GtkWidget *kontr, gpointer dane) {
 	tmp = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(data->window2)->entry), 0, -1) ;
 
 	if(sscanf(tmp, "%i -", &number) != 1) {
-		Warn("reading option combo failed. Sorry.") ;
+		say(0, "reading option combo failed. Sorry.") ;
 		g_free(tmp) ;
 		return ;
-	} else Warn("editing option %i", number) ;
+	} else say(0, "editing option %i", number) ;
 
 	o = g_list_nth_data(program->options, (number - 1)) ;
 	program->options = g_list_remove(program->options, o) ;
@@ -676,7 +640,7 @@ void programs_add_option(GtkWidget *kontr, gpointer dane) {
 	data_s *data ;
 	char *tmp ;
 
-	Warn("programs_add_option") ;
+	say(0, "programs_add_option") ;
 	/* casting provided data into apriopriate types */
 	data = (data_s *) dane ;
 
@@ -738,7 +702,7 @@ void programs_create_options(GtkWidget *kontr, gpointer dane) {
 		if(extract_program_from_entries(program, entries) == EXIT_FAILURE) return ;
 	}
 
-	Warn("programs_create_options: program name %s", program->name) ;
+	say(0, "programs_create_options: program name %s", program->name) ;
 
 	/* destroying the old function of next */
 	gtk_widget_destroy(GTK_WIDGET(next)) ;
@@ -837,14 +801,14 @@ int create_programs_dialog(opt_s *o, entry_s *program_template) {
 
 	for(i = 0 ; i < 10 ; i++) entries[i] = NULL ;
 
-	Warn("Creating program from template") ;
+	say(0, "create_programs_dialog: Creating program from template") ;
 	program = new_entry_s(program_template) ;
 
 	okno = gtk_window_new(GTK_WINDOW_DIALOG) ;
 	gtk_window_set_title(GTK_WINDOW(okno), "New program entry") ;
 	gtk_signal_connect(GTK_OBJECT(okno), "destroy", GTK_SIGNAL_FUNC(gtk_main_quit), okno) ;
 
-	Warn("programs_create") ;
+	say(0, "create_programs_dialog: programs_create") ;
 	nbook = gtk_notebook_new() ;
 
 	ypole = gtk_vbox_new(FALSE, 5) ;
@@ -884,7 +848,7 @@ int create_programs_dialog(opt_s *o, entry_s *program_template) {
 	g_main_iteration(FALSE) ;
 
 	if(program_template) {
-		Warn("program template non NULL!") ;
+		say(0, "program template non NULL!") ;
 		programs_create_options(NULL, &data) ;
 	}
 
@@ -911,7 +875,7 @@ void programs_edit_chooser(GtkWidget *kontr, gpointer dane) {
 	char *res ;
 
 	data = (data_s *) dane ;
-	Warn("programs_edit_chooser") ;
+	say(0, "programs_edit_chooser") ;
 	okno = data->window ;
 	combo = data->window1 ;
 
@@ -934,37 +898,49 @@ void programs_edit(gpointer dane, guint signal, GtkWidget *kontrolka) {
 	char *answer ;
 
 	o = (opt_s *) dane ;
+	data = g_malloc(sizeof(*data)) ;
+	data->data = NULL ;
 
 	/* make a nice GList */
+	say(0, "programs_edit: Making glist") ;
 	for(i = 0 ; i < g_list_length(o->program_list) ; i++) {
 		p = g_list_nth_data(o->program_list, i) ;
 		l = g_list_append(l, p->menupath) ;
+		say(0, "programs_edit: adding %s", p->menupath) ;
 	}
 
 	/* ...and a combo box out of it */
+	say(0, "programs_edit: Making combo") ;
 	combo = gtk_combo_new() ;
   gtk_combo_set_popdown_strings(GTK_COMBO(combo), l) ;
 
 	/* now pass the new widget to a somewhat more complex dialog */
 	data->window1 = combo ;
 
-	Warn("executing demand complex") ;
+	say(0, "executing demand complex") ;
 	demand_complex(combo, programs_edit_chooser, 
 		data, "Please choose the program to edit") ;
+	say(0, "returned from demand complex") ;
 
-	answer = (char*) data->data ;
-	Warn("your choice: %s", answer) ;
+	/* checking whether we got an answer */
+	if(data->data) {
+		answer = (char*) data->data ;
+		say(0, "your choice: %s", answer) ;
 
-	for(i = 0 ; i < g_list_length(o->program_list) ; i++) {
-		p = g_list_nth_data(o->program_list, i) ;
-		if(strcasecmp(p->menupath, answer) == 0) break ;
+		/* looking for the 'answer' in the program list */
+		for(i = 0 ; i < g_list_length(o->program_list) ; i++) {
+			p = g_list_nth_data(o->program_list, i) ;
+			if(strcasecmp(p->menupath, answer) == 0) break ;
+		}
+
+		if(i == g_list_length(o->program_list)) p = NULL ;
+
+		/* OK, let the editing begin */
+		create_programs_dialog(o, p) ;
 	}
 
-	if(i == g_list_length(o->program_list)) p = NULL ;
-
-	create_programs_dialog(o, p) ;
-
 	g_list_free(l) ;
+	g_free(data) ;
 }
 
 
@@ -977,7 +953,7 @@ int rebuild_menu(opt_s *o) {
 	entry_s *e ;
 	int i ;
 
-	Warn("rebuild_menu") ;
+	say(0, "rebuild_menu") ;
 
 	/* deleting menu entries from the list */
 	for(i = 0 ; i < g_list_length(o->program_list) ; i++) {
@@ -995,7 +971,7 @@ int rebuild_menu(opt_s *o) {
 		/* preparing item_factory item */
 		element.path = e->menupath ;
 		element.callback_action = i ;
-		Warn("path: %s entry: %s", e->menupath, e->entry) ;
+		say(0, "path: %s entry: %s", e->menupath, e->entry) ;
 
 		/* creating menu entry */
 		gtk_item_factory_create_items(o->fabryka, 1, &element, o) ;
@@ -1012,11 +988,11 @@ int delete_all_program_entries(opt_s *o) {
 	entry_s *program ;
 	char *t ;
 
-	Warn("%i programs to delete", g_list_length(o->program_list)) ;
+	say(0, "%i programs to delete", g_list_length(o->program_list)) ;
 
 	while(g_list_length(o->program_list) > 0) {
 		program = g_list_nth_data(o->program_list, 0) ;
-		Warn("deleting %s", program->menupath) ;
+		say(0, "deleting %s", program->menupath) ;
 		gtk_item_factory_delete_item(o->fabryka, program->menupath) ;
 
 		while( (t = strrchr(program->menupath, '/')) && t != program->menupath) {
@@ -1028,7 +1004,7 @@ int delete_all_program_entries(opt_s *o) {
 
 			t-- ;
 			*t = '\0' ;
-			/* Warn("deleting %s", program->menupath) ;
+			/* say(0, "deleting %s", program->menupath) ;
 			if( strcasecmp(program->menupath, "/Programs") != 0) 
 				gtk_item_factory_delete_item(o->fabryka, program->menupath) ;*/
 		}
@@ -1049,7 +1025,7 @@ void programs_delete_all(gpointer dane, guint signal, GtkWidget *kontrolka) {
 	o = (opt_s *) dane ;
 	if( !demand("Are you sure you want to delete all entries?") ) return ;
 
-	Warn("deleting entries...") ;
+	say(0, "deleting entries...") ;
 	delete_all_program_entries(o) ;
 }
 
@@ -1060,7 +1036,7 @@ void programs_reload_all(gpointer dane, guint signal, GtkWidget *kontrolka) {
 
 	o = (opt_s *) dane ;
 	if( !demand("Are you sure you want to reload all entries?") ) return ;
-	Warn("reloading programs") ;
+	say(0, "reloading programs") ;
 
 	load_program_files(o) ;
 }
@@ -1085,7 +1061,7 @@ int append_item_to_menu(opt_s *o, entry_s *e) {
 		tmp = g_list_nth_data(o->program_list, i) ;
 
 		if(strcasecmp(tmp->menupath, e->menupath) == 0) {
-			Warn("duplicate entry - %s - found, deleting predecessor", tmp->name) ;
+			say(0, "duplicate entry - %s - found, deleting predecessor", tmp->name) ;
 			o->program_list = g_list_remove(o->program_list, tmp) ;
 			gtk_item_factory_delete_item(o->fabryka, tmp->menupath) ;
 			free_entry_s(tmp, 0) ;
@@ -1160,14 +1136,14 @@ int load_program_files(opt_s *o) {
 
 	/* reading program description from the global file */
 	if( !(fp = fopen(ARKA_PROGRAMS, "r")) ) { 
-		Warn("File %s could not be opened", ARKA_PROGRAMS) ;
+		say(0, "File %s could not be opened", ARKA_PROGRAMS) ;
 	} else {
-		Warn("reading %s", ARKA_PROGRAMS) ;
+		say(0, "reading %s", ARKA_PROGRAMS) ;
 		i = parse_and_append(o, fp, i) ;
 	}
 
 	if( !(rcfile = my_strdup(getenv("HOME")) ) ) {
-		Warn("$HOME undefined.") ;
+		say(0, "$HOME undefined.") ;
 		return EXIT_FAILURE ;
 	}
 
@@ -1177,13 +1153,13 @@ int load_program_files(opt_s *o) {
 	strcat(rcfile, "/programs") ;
 
 	if( !(fp = fopen(rcfile, "r")) ) { 
-		Warn("File %s could not be opened", rcfile) ;
+		say(0, "File %s could not be opened", rcfile) ;
 	} else {
-		Warn("reading %s", rcfile) ;
+		say(0, "reading %s", rcfile) ;
 		i = parse_and_append(o, fp, i) ;
 	}
 
-	Warn("%i programs loaded", g_list_length(o->program_list)) ;
+	say(0, "%i programs loaded", g_list_length(o->program_list)) ;
 
 	return EXIT_SUCCESS ;
 }
@@ -1208,11 +1184,11 @@ void program_select(gpointer dane, guint signal, GtkWidget *kontrolka) {
 	o = (opt_s *) dane ;
 	cmdline = set_options(signal, o) ; /* do the dialog stuff */
 
-	Warn("program_select: got: %s\n from set_options", cmdline) ;
+	say(0, "program_select: got: %s\n from set_options", cmdline) ;
 
 	if(!cmdline) return ;
 
-	Warn("program_select: executing\n%s", cmdline) ;
+	say(0, "program_select: executing\n%s", cmdline) ;
 	o->command_history = g_list_prepend(o->command_history, cmdline) ;
 
 	execute(cmdline, o) ;
@@ -1258,7 +1234,7 @@ void close_options(GtkWidget *kontr, gpointer dane) {
 	p = (entry_s *) d->data1 ;
 
 	res = malloc(2000) ; /* yeah I know this is evil */
-	Warn("close_options: chosen program is %s", p->name) ;
+	say(0, "close_options: chosen program is %s", p->name) ;
 	sprintf(res, "%s ", p->name) ;
 
 	/* for each option... */
@@ -1330,7 +1306,7 @@ GtkWidget * set_widget_for_option(entry_option_s *o, GtkWidget *parentbox) {
 	res = NULL ;
 
 	hpole = gtk_hbox_new(FALSE, 5) ;
-	gtk_box_pack_start(GTK_BOX(parentbox), hpole, FALSE, FALSE, 2) ;
+	gtk_box_pack_start(GTK_BOX(parentbox), hpole, FALSE, FALSE, 0) ;
 
 	switch(o->type){
 
@@ -1396,7 +1372,7 @@ GtkWidget * set_widget_for_option(entry_option_s *o, GtkWidget *parentbox) {
 		case HIDDEN:
 			break ;
 		default:
-			Warn("unsupported option!") ;
+			say(0, "unsupported option!") ;
 			break ;
 	}
 
@@ -1428,7 +1404,7 @@ char* set_options(int wh, opt_s *main_opts) {
 	d.data = NULL ;
 	d.data1 = p ;
 
-	Warn("set_options: selection %i, program name %s", wh, p->name) ;
+	say(0, "set_options: selection %i, program name %s", wh, p->name) ;
 
 	/* main option window */
 	okno = gtk_window_new(GTK_WINDOW_DIALOG) ;
@@ -1439,13 +1415,13 @@ char* set_options(int wh, opt_s *main_opts) {
 	d.window1 = okno ;
 
 	label = gtk_label_new("Main") ;
-	ypole = gtk_vbox_new(FALSE, 5) ;
+	ypole = gtk_vbox_new(FALSE, 0) ;
 	notebook = gtk_notebook_new() ;
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), ypole, label) ;
 	gtk_container_add(GTK_CONTAINER(okno), notebook) ;
 
 	if(p->long_description) {
-		label = gtk_label_new("More..") ;
+		label = gtk_label_new("Description") ;
 		tmp = gtk_label_new(p->long_description) ;
 		gtk_label_set_justify(GTK_LABEL(tmp), GTK_JUSTIFY_LEFT) ;
 		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp, label) ;
@@ -1476,9 +1452,9 @@ char* set_options(int wh, opt_s *main_opts) {
 	hpole = create_hbox(ypole) ;
 
 	/* ok and cancel buttons */
-	ok = create_button(hpole, close_options, "OK", &d) ;
-	cancel = create_button(hpole, zakoncz_dialog, "Cancel", okno) ;
-	man = create_button(hpole, help_display_man, "Man page", p->name) ;
+	ok = create_button(hpole, close_options, " Launch ", &d) ;
+	cancel = create_button(hpole, zakoncz_dialog, " Cancel ", okno) ;
+	man = create_button(hpole, help_display_man, " Man page ", p->name) ;
 
 	GTK_WIDGET_SET_FLAGS(ok, GTK_CAN_DEFAULT) ;
 	gtk_widget_grab_default(ok) ;
@@ -1500,7 +1476,7 @@ int entry_print(entry_s *e) {
 	int i ;
 	entry_option_s *o ;
 
-	Warn("printing entry") ;
+	say(0, "printing entry") ;
 	printf("%s - %s\n\n", e->entry, e->short_description) ;
 	printf("Usage:\n\t%s ", e->name) ;
 	for(i = 0 ; i < g_list_length(e->options) ; i++) {
@@ -1687,7 +1663,7 @@ int parse_option(char *line, entry_s *e, int i) {
 		else if(strcmp(p, "file") == 0) o->type = FILENAME ;
 		else if(strcmp(p, "string") == 0) o->type = STRING ;
 		else if(strcmp(p, "invisible") == 0) o->type = HIDDEN ;
-		else Warn("bad option in line %i: %s", i, p) ;
+		else say(0, "bad option in line %i: %s", i, p) ;
 	}
 
 	/* reading default value */
