@@ -19,7 +19,7 @@ GList * get_items_from_file(char *file) {
 	int i = 0 ;
 
 	if( !(fp = fopen(file, "r")) ) {
-		Warn("cannot open %s", file) ;
+		say(0, "cannot open %s", file) ;
 		return NULL ;
 	}
 
@@ -35,7 +35,7 @@ GList * get_items_from_file(char *file) {
 		res = g_list_append(res, a) ;
 	}
 
-	Warn("%i entries read from file %s", i, file) ;
+	say(0, "%i entries read from file %s", i, file) ;
 	fclose(fp) ;
 	return res ;
 }
@@ -49,11 +49,11 @@ int put_items_to_file(char *file, GList *list, int items) {
 
 	/* open the file */
 	if( !(fp = fopen(file, "w")) ) {
-		Warn("Could not open file %s for writing", file) ;
+		say(0, "Could not open file %s for writing", file) ;
 		return EXIT_FAILURE ;
 	}
 
-	Warn("....") ;
+	say(0, "....") ;
 
 	/* how many *last* items to write. (items == 0) => all */
 	l = g_list_length(list) ;
@@ -67,7 +67,7 @@ int put_items_to_file(char *file, GList *list, int items) {
 		fprintf(fp, "%s\n", tmp) ;
 	}
 
-	Warn("put_items_to_file: wrote %i items", k) ;
+	say(0, "put_items_to_file: wrote %i items", k) ;
 	fclose(fp) ;
 	return EXIT_SUCCESS ;	
 }
@@ -79,6 +79,8 @@ char *find_key(char *line, char *key) {
 
 	res = NULL ;
 	while(isspace(*line)) line++ ;
+	
+	if( (tmp = strchr(line, '\n')) ) *tmp = '\0' ;
 
 	if( (tmp = strstr(line, key)) ) {
 		res = my_strdup(tmp + strlen(key)) ;
@@ -92,13 +94,14 @@ char *find_key(char *line, char *key) {
 int read_options(char *fname, opt_s *o) {
 	char bufor[BUFSIZ], *tmp, *val, *p ;
 	FILE *fp ;
+	int i ;
 
 	if( !(fp = fopen(fname, "r")) ) {
-		Warn("cannot open file %s", fname) ;
+		say(0, "cannot open file %s", fname) ;
 		return EXIT_FAILURE ;
 	}
 	
-	Warn("reading options from file %s", fname) ;
+	say(0, "reading options from file %s", fname) ;
 	while( (tmp = fgets(bufor, BUFSIZ, fp)) ) {
 
 		while(isspace(*tmp)) tmp++ ;
@@ -108,30 +111,40 @@ int read_options(char *fname, opt_s *o) {
 
 		if( (val = find_key(tmp, "file_history_size=")) ) {
 			o->file_history_size = atoi(val) ;
-			Warn("file_history_size = %i", o->file_history_size) ;
+			say(0, "file_history_size = %i", o->file_history_size) ;
 			free(val) ;
 		}
 
 		if( (val = find_key(tmp, "hide_tooltips=")) ) {
-			o->show_tooltips = !atoi(val) ;
-			Warn("show_tooltips = %i", o->show_tooltips) ;
+			i = !atoi(val) ;
+			if(i != o->show_tooltips) {
+				say(0, "show_tooltips status changed") ;
+				o->show_tooltips = i ;
+			}
+
+			say(0, "show_tooltips = %i", o->show_tooltips) ;
 			free(val) ;
 		}
 
 		if( (val = find_key(tmp, "command_history_size=")) ) {
 			o->command_history_size = atoi(val) ;
-			Warn("command_history_size = %i", o->command_history_size) ;
+			say(0, "command_history_size = %i", o->command_history_size) ;
 			free(val) ;
 		}
 
 		if( (val = find_key(tmp, "font_name_default=")) ) {
 			if(o->font_name_default) free(o->font_name_default) ;
 			o->font_name_default = val ;
-			Warn("font_name_default = %s", o->font_name_default) ;
+			say(0, "font_name_default = %s", o->font_name_default) ;
 			if(o->font_name) free(o->font_name) ;
 			o->font_name = my_strdup(val) ;
 			if( !(o->czcionka = gdk_font_load(o->font_name)) )
-				Warn("Cannot load font %s", o->font_name) ;
+				say(0, "Cannot load font %s", o->font_name) ;
+		}
+
+		if( (val = find_key(tmp, "external_editor=")) ) {
+			if(o->external_editor) free(o->external_editor) ;
+			o->external_editor = val ;
 		}
 
 	}
@@ -145,7 +158,7 @@ int write_options(char *file, opt_s *o) {
 	FILE *fp ;
 
 	if( !(fp = fopen(file, "w")) ) {
-		Warn("file %s cannot be opened", file) ;
+		say(0, "file %s cannot be opened", file) ;
 		return EXIT_FAILURE ;
 	}
 
@@ -156,6 +169,7 @@ int write_options(char *file, opt_s *o) {
 	fprintf(fp, "command_history_size=%i\n", o->command_history_size) ;
 	fprintf(fp, "font_name_default=%s\n", o->font_name_default) ;
 	fprintf(fp, "hide_tooltips=%i\n", !(o->show_tooltips)) ;
+	fprintf(fp, "external_editor=%s\n", o->external_editor) ;
 
 	fclose(fp) ;
 	return EXIT_SUCCESS ;
@@ -168,7 +182,7 @@ int write_rcfile(opt_s *o) {
 	char *rcfile, *rcdir ;
 
 	if(! (rcdir = my_strdup(getenv("HOME"))) ) {
-		Warn("$HOME variable absent") ;
+		say(0, "$HOME variable absent") ;
 		return EXIT_FAILURE ;
 	}
 	
@@ -181,10 +195,10 @@ int write_rcfile(opt_s *o) {
 	strcat(rcfile, "/command_history") ;
 
 	if(o->command_history) {
-		Warn("saving command history...") ;
+		say(0, "saving command history...") ;
 		put_items_to_file(rcfile, o->command_history, o->command_history_size) ;
 	} else {
-		Warn("no command history recorded") ;
+		say(0, "no command history recorded") ;
 	}
 
 	/* saves file history */
@@ -192,10 +206,10 @@ int write_rcfile(opt_s *o) {
 	strcat(rcfile, "/file_history") ;
 
 	if(o->file_history) {
-		Warn("saving file history...") ;
+		say(0, "saving file history...") ;
 		put_items_to_file(rcfile, o->file_history, o->file_history_size) ;
 	} else {
-		Warn("no file history recorded") ;
+		say(0, "no file history recorded") ;
 	}
 
 	/* saving options */
@@ -204,9 +218,13 @@ int write_rcfile(opt_s *o) {
 
 	write_options(rcfile, o) ;
 
+	strcpy(rcfile, rcdir) ;
+	strcat(rcfile, "/shortcuts") ;
+	gtk_item_factory_dump_rc(rcfile, NULL, TRUE) ;
+
 	free(rcdir) ;
 	free(rcfile) ;
-	Warn("configuration saved succesfully") ;
+	say(0, "configuration saved succesfully") ;
 	return EXIT_SUCCESS ;
 }
 
@@ -216,11 +234,11 @@ int read_rcfile(opt_s *o) {
 	char *rcfile, *rcdir ;
 	struct stat statbuf ;
 
-	Warn("reading configuration") ;
+	say(0, "reading configuration") ;
 
 	/* getting rc directory */
 	if(! (rcdir = my_strdup(getenv("HOME"))) ) {
-		Warn("$HOME not present. No configuration file read") ;
+		say(0, "$HOME not present. No configuration file read") ;
 		return EXIT_FAILURE ;
 	}
 
@@ -252,7 +270,7 @@ int read_rcfile(opt_s *o) {
 		}
 	}
 
-	Warn("configuration directory %s found", rcdir) ;
+	say(0, "configuration directory %s found", rcdir) ;
 	rcfile = malloc(strlen(rcdir) + 50) ;
 
 	/* reading command history */
@@ -276,9 +294,13 @@ int read_rcfile(opt_s *o) {
 
 	read_options(rcfile, o) ;
 
+	strcpy(rcfile, rcdir) ;
+	strcat(rcfile, "/shortcuts") ;
+	gtk_item_factory_parse_rc(rcfile) ;
+
 	free(rcdir) ;
 	free(rcfile) ;
-	Warn("configuration read succesfully") ;
+	say(0, "configuration read succesfully") ;
 	
 	return EXIT_SUCCESS ;
 }
